#include "factory.h"
#include "factory.cxx"
#include "product.h"

using std::cout;
using std::endl;
using std::copy;
using std::map;
using std::ostream_iterator;
using std::ostream;
using std::set;
using std::string;


class MyLoadStrategy : public regime::load_strategy<int> {
  string m_lib_extension;
public:
  MyLoadStrategy(const string& ext) {
        m_lib_extension = ext;
  }
  virtual string find(const int& key) {
    if (key < 0)
      return "";
    // I would use stringstream here, but it is not yet
    // available in my implementation of C++ (egcs-1.1.2)
    char buf[256];
    sprintf(buf, "product%d.%s", key, m_lib_extension.c_str());
    string result(buf);
    cout<<"Should load library: "<<result<<endl;
    return result;
  }
};

typedef regime::factory<int,MyBase> IntFactory;

void dump_libraries()
{
  cout<<"IntFactory keys: ";
  const IntFactory::keys& ikeys = IntFactory::query_keys();
  copy(ikeys.begin(), ikeys.end(), ostream_iterator<int>(cout, " "));
  cout<<endl;

  cout<<"IntFactory libraries:"<<endl;
  const IntFactory::libraries& ilibs = IntFactory::query_libraries();
  copy(ilibs.begin(), ilibs.end(), 
    ostream_iterator<IntFactory::libraries::value_type>(cout));
}

void KAPI run_kernel(const char libname[])
{
  IntFactory::load_library(libname);
  const char* extension = strchr(libname,'.')+1;
  IntFactory::set_load_strategy(new MyLoadStrategy(extension));

  cout<<"----------------------- before creating ----------------"<<endl;
  dump_libraries();
  cout<<"--------------------------------------------------------"<<endl;

  MyBase::FactoryParams pm;
  pm.a1 = "hello product";
  MyBase* b1 = IntFactory::create(1, pm);
  if (b1 == 0)
    cout<<"product with key '1' cannot be created"<<endl;
  else
    cout<<"product with key '1' says:"<<*b1<<endl;

  MyBase* b2 = IntFactory::create(2);
  if (b2 == 0)
    cout<<"product with key '2' cannot be created"<<endl;
  else
    cout<<"product with key '2' says:"<<*b2<<endl;

  cout<<"------------------------ after creating ----------------"<<endl;
  dump_libraries();
  cout<<"--------------------------------------------------------"<<endl;

  IntFactory::unload_library(libname);

  cout<<"------------------------ after unloading ---------------"<<endl;
  dump_libraries();
  cout<<"--------------------------------------------------------"<<endl;
}



